#include "LPC13xx.h"			/* LPC13xx Peripheral Registers */
#include "type.h"
#include "i2c.h"
#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "usbcore.h"
#include "cdc.h"
#include "cdcuser.h"
#include "serial.h"
#include "uart.h"

typedef struct {
uint8_t d[25];
int num;
}GETDAT_T;

#define     EN_TIMER32_1    (1<<10)
#define     EN_IOCON        (1<<16)
#define     EN_USBREG       (1<<14)
#define     EN_TIMER16_1    (1<<2)

GETDAT_T USBdata;
GETDAT_T UARTdata;
extern volatile uint32_t I2CCount;
extern volatile uint8_t I2CMasterBuffer[BUFSIZE];
extern volatile uint8_t I2CSlaveBuffer[BUFSIZE];
extern volatile uint32_t I2CMasterState;
extern volatile uint32_t I2CReadLength, I2CWriteLength;
uint8_t timestamp = 0;
uint8_t Asa[3];
/*----------------------------------------------------------------------------
 Initializes the VCOM port.
 Call this function before using VCOM_putchar or VCOM_getchar
 *---------------------------------------------------------------------------*/
void VCOM_Init(void)
{
	CDC_Init ();
}
/*----------------------------------------------------------------------------
  Reads character from serial port buffer and writes to USB buffer
 *---------------------------------------------------------------------------*/
void VCOM_Serial2Usb(GETDAT_T dat)
{
	if (dat.num!=0 && CDC_DepInEmpty) { // 1 is empty so if empty
		CDC_DepInEmpty = 0;
		USB_WriteEP (CDC_DEP_IN, (unsigned char* )&dat.d[0], dat.num);
	}
}
/*----------------------------------------------------------------------------
  Reads character from USB buffer and writes to serial port buffer
 *---------------------------------------------------------------------------*/
void VCOM_Usb2Serial(void)
{
  static char serBuf [32];
         int  numBytesToRead, numBytesRead, numAvailByte;

  CDC_OutBufAvailChar (&numAvailByte);
  if (numAvailByte > 0) {
      numBytesToRead = numAvailByte > 32 ? 32 : numAvailByte;
      numBytesRead = CDC_RdOutBuf (&serBuf[0], &numBytesToRead);
      ser_Write (&serBuf[0], &numBytesRead);
  }
}
/*----------------------------------------------------------------------------
  checks the serial state and initiates notification
 *---------------------------------------------------------------------------*/
void VCOM_CheckSerialState (void)
{
	unsigned short temp;
	static unsigned short serialState;

	temp = CDC_GetSerialState();
	if (serialState != temp)
	{
		serialState = temp;
		CDC_NotificationIn();                  // send SERIAL_STATE notification
	}
}
/*----------------------------------------------------------------------------
  Check
 *---------------------------------------------------------------------------*/
void Start_Wait(int time)
{
	LPC_TMR16B1->MR0 = time; // time
	LPC_TMR16B1->TCR = 0x02; // TCR Reset
	LPC_TMR16B1->TCR = 1; // TCR Start
}
/*----------------------------------------------------------------------------
  Check
 *---------------------------------------------------------------------------*/
void Stop_Wait()
{

	while (LPC_TMR16B1->TCR & 0x01); //wait

	return;
}
/*----------------------------------------------------------------------------
  MPU6050 (AccXYZ + GyroXYZ)
 *---------------------------------------------------------------------------*/
uint8_t I2C_MPU6050 (void)
{
	volatile uint32_t i;

	I2CWriteLength = 2;
	I2CReadLength = 14;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x3B; //Address 14byte (AccXYZ + GyroXYZ)
	I2CMasterBuffer[2] = MPU6050_R;
	I2CEngine();

	for ( i = 3; i < 17; i++ )
	{
		if((i & 0x01) == 0x01)
		{
			USBdata.d[i + 1] = I2CSlaveBuffer[i - 3];
		}
		else
		{
			USBdata.d[i - 1] = I2CSlaveBuffer[i - 3];
		}
	}
	return 0;
}
/*----------------------------------------------------------------------------
  AK8975 (MagXYZ)
 *---------------------------------------------------------------------------*/
uint8_t I2C_AK8975 (void)
{
	volatile uint32_t i;
	int16_t Mag[3];
//	uint8_t sign;

	I2CWriteLength = 2;
	I2CReadLength = 6;
	I2CMasterBuffer[0] = AK8975_W;
	I2CMasterBuffer[1] = 0x03; //Address 6byte (MagXYZ)
	I2CMasterBuffer[2] = AK8975_R;
	I2CEngine();

	for ( i = 0; i < 3; i++ )
	{
		Mag[i] = (uint16_t)I2CSlaveBuffer[i<<1] | (((uint16_t)I2CSlaveBuffer[(i<<1)+1])<<8);

		Mag[i] = Mag[i]*(uint16_t)((((int8_t)Asa[i] - 128)/256) + 1);
	}

	USBdata.d[17] = (uint8_t)Mag[1];
	USBdata.d[18] = (uint8_t)(Mag[1]>>8);

	USBdata.d[19] = (uint8_t)Mag[0];
	USBdata.d[20] = (uint8_t)(Mag[0]>>8);

	Mag[2] = ~Mag[2];
	USBdata.d[21] = (uint8_t)Mag[2];
	USBdata.d[22] = (uint8_t)(Mag[2]>>8);

	Start_Wait(10);
	Stop_Wait();

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = AK8975_W;
	I2CMasterBuffer[1] = 0x0A; //Address start MagXYZ ADC (need 7.2ms)
	I2CMasterBuffer[2] = 0x01;
	I2CEngine();
	return 0;
}
/*----------------------------------------------------------------------------
  Init MPU6050
 *---------------------------------------------------------------------------*/
void Init_MPU6050 (void)
{
	volatile uint32_t i;

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x6B; //Address start MPU9150
	I2CMasterBuffer[2] = 0x00;
	I2CEngine();

	Start_Wait(10);
	Stop_Wait();

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x37; //Address auxiliary I2C
	I2CMasterBuffer[2] = 0x02;
	I2CEngine();

	Start_Wait(10);
	Stop_Wait();

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1A; //Address digital low pass filter
	I2CMasterBuffer[2] = 0x02; //100Hz
	I2CEngine();

	Start_Wait(10);
	Stop_Wait();

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1B; //range ang
	I2CMasterBuffer[2] = 0x18; //2000deg/s
	I2CEngine();

	Start_Wait(10);
	Stop_Wait();

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1C; //range acc
	I2CMasterBuffer[2] = 0x18; //16g
	I2CEngine();
}
/*----------------------------------------------------------------------------
  Init AK8975
 *---------------------------------------------------------------------------*/
void Init_AK8975 (void)
{
	volatile uint32_t i;

	I2CWriteLength = 2;
	I2CReadLength = 3;
	I2CMasterBuffer[0] = AK8975_W;
	I2CMasterBuffer[1] = 0x10; //Address 3byte (adjust MagXYZ)
	I2CMasterBuffer[2] = AK8975_R;
	I2CEngine();

	for(i = 0;i < 3;i ++)
	{
		Asa[i] = I2CSlaveBuffer[i];
	}
}
/*----------------------------------------------------------------------------
  Init Timer32
 *---------------------------------------------------------------------------*/
void Init_Timer32_1 (void)
{
	NVIC_EnableIRQ(TIMER_32_1_IRQn);
	NVIC_SetPriority(TIMER_32_1_IRQn, 2);
	LPC_TMR32B1->PR =7200-1; // 10kHz(Max 32bit dec:4294967295)
	LPC_TMR32B1->MCR =0x600; // MR3 as Period of 32B1 and interrupt
	LPC_TMR32B1->MR3 = 100; // PWM Period 10k/100=100Hz
	LPC_TMR32B1->TCR =2; // TCR Reset
}
/*----------------------------------------------------------------------------
  Timer32bit
 *---------------------------------------------------------------------------*/
void TIMER32_1_IRQHandler(void)
{
	volatile uint32_t i;

	LPC_TMR32B1->IR=0x08; //clear interrupt flag

	for ( i = 0; i < BUFSIZE; i++ ) //clear I2CSlaveBuffer
	{
			I2CSlaveBuffer[i] = 0x00;
	}

	USBdata.num = 3 + 14 + 6;
	USBdata.d[0] = USBdata.d[1] = 0xff;
	USBdata.d[2] = timestamp;

	if(timestamp == 0xff)
	{
		timestamp = 0;
	}
	else
	{
		timestamp++;
	}

	I2C_MPU6050();

	for ( i = 0; i < BUFSIZE; i++ ) //clear I2CSlaveBuffer
	{
			I2CSlaveBuffer[i] = 0x00;
	}

	Start_Wait(10);
	Stop_Wait();

	I2C_AK8975();

	UARTdata = USBdata;
	LPC_UART->IER = IER_THRE | IER_RLS;			/* Disable RBR */
	UARTSend( (uint8_t *)UARTdata.d, UARTdata.num );
	LPC_UART->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
	VCOM_Serial2Usb(USBdata); // read serial port and initiate USB event
	VCOM_CheckSerialState();
}
/*----------------------------------------------------------------------------
  Init IO
 *---------------------------------------------------------------------------*/
void Init_IO (void)
{
	LPC_IOCON->PIO2_0 &= ~0xc0; //Init LED
	LPC_GPIO2->DIR |= 0x01;
	LPC_GPIO2->DATA &= ~0x01; //LED off
}
/*----------------------------------------------------------------------------
  Check
 *---------------------------------------------------------------------------*/
/*
void Check_address (void)
{
	I2CWriteLength = 2;
	I2CReadLength = 1;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1B;
	I2CMasterBuffer[2] = MPU6050_R;
	I2CEngine();

	USBdata.d[USBdata.num + 3] = I2CSlaveBuffer[0];
	USBdata.num = 4;

	VCOM_Serial2Usb(USBdata); // read serial port and initiate USB event
	VCOM_CheckSerialState();
}
*/
/*----------------------------------------------------------------------------
  Main
 *---------------------------------------------------------------------------*/
int main (void)
{
	volatile uint32_t i;

	LPC_SYSCON->SYSAHBCLKCTRL |= (EN_TIMER16_1 | EN_TIMER32_1 | EN_IOCON | EN_USBREG);

	Init_IO(); // IO Initialization
	USBIOClkConfig();
	LPC_TMR16B1->PR = 7200-1; // 10kHz (Max 16bit dec:65535)
	LPC_TMR16B1->MCR = 0x04; // Stop timer on MR0
	Init_Timer32_1(); // TIMER32_1 Initialization
	VCOM_Init(); // VCOM Initialization
	USB_Init(); // USB Initialization
	UARTInit(115200);
	USB_Connect(TRUE); // USB Connect

	Start_Wait(100);

	while ((!USB_Configuration) || (LPC_TMR16B1->TCR & 0x01)); // wait until USB is configured
	while (!I2CInit((uint32_t)I2CMASTER)); // wait until I2C is configured

	Init_MPU6050(); // MPU6050 Initialization
	Init_AK8975(); // AK8975 Initialization

	LPC_TMR32B1->TCR = 1; // TCR start

	while(1) //Infinite loop
	{
		i++;
	}
	return 0;
}
/*----------------------------------------------------------------------------
  End
 *---------------------------------------------------------------------------*/
