import processing.serial.*;
import processing.opengl.*;

Serial port; 
int[] inByte = new int[100];
int[] ad_new = new int[100];
int[] ad_base = new int[100];

float[] omega_vec         = {0.0, 0.0, 0.0};           //角速度ベクトル (x,y,z)[rad]
float[] acc_vec           = {0.0, 0.0, 0.0};           //加速度ベクトル (x,y,z) [g]
float[] mag_vec           = {0.0, 0.0, 0.0};           //地磁気ベクトル (x,y,z) [uT]

float   acc_norm          = 0.0;                       //加速度ベクトルのノルム 
float   mag_norm          = 0.0;                       //地磁気ベクトルのノルム
float   mag_cor_norm      = 0.0;                       //補正した地磁気ベクトルのノルム

//リファレンスの取得
void ad_base()
{
  int count = 0;
  float[] adc = new float[100];
  while (count < 100)
  {
    for (int i = 3; i < 16; i = i + 2)
    {
      adc[i] += (float)(inByte[i] + (inByte[i+1]<<8))/100.0;
      if (adc[i] > (int)(32767.0/100.0))
      {
        adc[i] = adc[i] - 65535.0/100.0;
      }
    }
    count++;
    delay(10);
  }
  for (int i = 3; i < 16; i = i + 2)
  {
    ad_base[i] = (int)adc[i];
  }
}

void setup() 
{
  size(650, 600, P3D);
  frameRate(60);
  println(Serial.list());
  port = new Serial(this, "COM3", 115200);  // select port
  ad_base();

}

void draw()
{
  background(0);  
  writeSenValue();
  draw3DGraphs();
}

void serialEvent(Serial p)
{
  if (port.available() > 22)
  {
    for (int i = 0; i < 23; i ++)
    {
      inByte[i] = port.read();
    }
    port.clear();

    for (int i = 3; i < 23; i = i + 2)
    {
      ad_new[i] = inByte[i] + (inByte[i+1]<<8);
      if (ad_new[i] >= 32767)
      {
        ad_new[i] = ad_new[i] - 65536;
      }
    }
  }
    //センサ値を角加速度ベクトルに変換
    omega_vec[0] = radians(((float)(ad_new[11] - ad_base[11] ) )/16.4);
    omega_vec[1] = radians(((float)(ad_new[13] - ad_base[13] ) )/16.4);
    omega_vec[2] = radians(((float)(ad_new[15] - ad_base[15] ) )/16.4);
    
    //センサ値を加速度ベクトルに変換
    acc_vec[0]   = (ad_base[3] - ad_new[3])/2048.0; 
    acc_vec[1]   = (ad_base[5] - ad_new[5])/2048.0; 
    acc_vec[2]   = (ad_base[7] - ad_new[7])/2048.0;  
    acc_norm = sqrt(acc_vec[0]*acc_vec[0]+acc_vec[1]*acc_vec[1]+acc_vec[2]*acc_vec[2]);
    //センサ値を地磁気ベクトルに変換
    mag_vec[0]   = ad_new[17] * 0.3;  
    mag_vec[1]   = ad_new[19] * 0.3; 
    mag_vec[2]   = ad_new[21] * 0.3;
    mag_norm = sqrt(mag_vec[0]*mag_vec[0]+mag_vec[1]*mag_vec[1]+mag_vec[2]*mag_vec[2]);
}

void writeSenValue()
{
  fill(255, 255, 255);
  textSize(20);
  text("accX[g]", 10, 20);
  text("accY[g]", 10, 40);
  text("accZ[g]", 10, 60);
  text("|a|=", 10, 80);
  text(acc_vec[0], 90, 20);
  text(acc_vec[1], 90, 40);
  text(acc_vec[2], 90, 60); 
  text(acc_norm, 60, 80); 

  text("magX[uT]", 180, 20);
  text("magY[uT]", 180, 40);
  text("magZ[uT]", 180, 60);
  text("|m|=", 180, 80);
  text(mag_vec[0], 300, 20);
  text(mag_vec[1], 300, 40);
  text(mag_vec[2], 300, 60);  
  text(mag_norm, 230, 80);  

  text("omegaX[deg/s]", 400, 20);
  text("omegaY[deg/s]", 400, 40);
  text("omegaZ[deg/s]", 400, 60);
  text(degrees(omega_vec[0]), 550, 20);
  text(degrees(omega_vec[1]), 550, 40);
  text(degrees(omega_vec[2]), 550, 60); 
  
}


void draw3DGraphs()
{
  //地磁気と加速度センサのベクトルを描画
  stroke(0, 0, 0);
  pushMatrix();
  translate(width/2, height/2);    
  rotateY(radians(-20));
  rotateX(radians(20));
  fill(255, 0, 0);
  box(200, 5, 5);   
  fill(0, 255, 0);
  box(5, 200, 5);
  fill(0, 0, 255);
  box(5, 5, 200);
  fill(100, 100, 100, 80);  

  for (int i=1; i<=40; i++) {
    pushMatrix();
    translate(acc_vec[0] *50 *i/20 , -acc_vec[2] *50*i/20, -acc_vec[1] *50*i/20 );
    fill(255, 55, 0);
    box(5, 5, 5);
    popMatrix();
  }

  for (int i=1; i<=40; i++) {
    pushMatrix();
    translate(mag_vec[0]  *i/20, -mag_vec[2] *i/20, -mag_vec[1] *i/20 );
    fill(200, 50, 200);
    box(5, 5, 5);
    popMatrix();
  }
  popMatrix();
 
}






