import processing.serial.*;
import processing.opengl.*;

Serial port; 
int[] inByte = new int[100];
int[] ad_new = new int[100];
int[] ad_base = new int[100];

float[] magdrawX = new float[501]; 
float[] magdrawY = new float[501];
float[] magdrawZ = new float[501];

float[] omega_vec         = {0.0, 0.0, 0.0};           //角速度ベクトル (x,y,z)[rad]
float[] acc_vec           = {0.0, 0.0, 0.0};           //加速度ベクトル (x,y,z) [1G]
float[] mag_vec           = {0.0, 0.0, 0.0};           //地磁気ベクトル (x,y,z) [uT]
float[] mag_corrected_vec = {0.0, 0.0, 0.0};           //中心の補正をした地磁気ベクトル　
float[] estimated_sphere_param = {0.0, 0.0, 0.0, 0.0}; //地磁気ベクトルの補正データ格納用
float[] acc_LPF_vec       = {0.0, 0.0, 0.0};           // 加速度ベクトルをLPFにかけたもの
float[] mag_cor_LPF_vec   = {0.0, 0.0, 0.0};           //補正した地磁気ベクトルをLPFにかけたもの

float   acc_norm          = 0.0;                       //加速度ベクトルのノルム 
float   mag_norm          = 0.0;                       //地磁気ベクトルのノルム
float   mag_cor_norm      = 0.0;                       //補正した地磁気ベクトルのノルム

float[] quat_3DOF         = {1.0, 0.0, 0.0, 0.0};      //現在姿勢のクオータニオン
float[] quat_6DOF         = {1.0, 0.0, 0.0, 0.0};
float[] quat_9DOF         = {1.0, 0.0, 0.0, 0.0};

float[][] rotMatrix_3DOF  = {{0.0, 0.0, 0.0},          //現在姿勢の姿勢行列
                             {0.0, 0.0, 0.0},
                             {0.0, 0.0, 0.0}
                            };     

float[][] rotMatrix_6DOF  = {{0.0, 0.0, 0.0},      
                             {0.0, 0.0, 0.0},
                             {0.0, 0.0, 0.0}
                            };     


float[][] rotMatrix_9DOF  = {{0.0, 0.0, 0.0},      
                             {0.0, 0.0, 0.0},
                             {0.0, 0.0, 0.0}
                            };     

float[] ZYX_euler_3DOF    =  {0.0, 0.0, 0.0};         //ジャイロの出力から求めたZYXオイラー角
float[] ZYX_euler_6DOF    =  {0.0, 0.0, 0.0};         //ジャイロの出力と加速度センサから求めたZYXオイラー角
float[] ZYX_euler_9DOF    =  {0.0, 0.0, 0.0};         //ジャイロ+加速度+地磁気のZYXオイラー角

float pitch_accmag = 0.0, roll_accmag = 0.0, yaw_accmag = 0.0; //加速度センサ, 地磁気センサから求めたZYXオイラー角

float inclination =  8.0;     //偏角(東京) 日本列島全体で6から10度
float declination =  49.0;    //伏角(東京) 日本列島全体で44から60度
                              //地磁気大きさ 範囲 24-66nT 東京付近の平均は45nT, 日本全体での平均は46nT

float dt   = 0.01;            //積分の刻み幅   

float gain_comp = 0.999;      //相補フィルタのゲイン
float gain_LPF    = 0.8;      //ローパスフィルタのゲイン

int[] valsA = new int[512];
int[] valsB = new int[512];
int[] valsC = new int[512];

int[] valsD = new int[512];
int[] valsE = new int[512];
int[] valsF = new int[512];

int[] valsG = new int[512];
int[] valsH = new int[512];
int[] valsI = new int[512];

int[] valsJ = new int[512];
int[] valsK = new int[512];
int[] valsL = new int[512];


int drawheight = 1050;

///////////////////
Thread thread1; // thread for timer
///////////////////

//リファレンスの取得
void ad_base()
{
  int count = 0;
  float[] adc = new float[100];
  while (count < 100)
  {
    for (int i = 3; i < 16; i = i + 2)
    {
      adc[i] += (float)(inByte[i] + (inByte[i+1]<<8))/100.0;
      if (adc[i] > (int)(32767.0/100.0))
      {
        adc[i] = adc[i] - 65535.0/100.0;
      }
    }
    count++;
    delay(10);
  }
  for (int i = 3; i < 16; i = i + 2)
  {
    ad_base[i] = (int)adc[i];
  }
}

void setup() 
{
  size(2100, 1100, P3D);
  frameRate(60);
  println(Serial.list());
  port = new Serial(this, "COM3", 115200);  // select port
  ad_base();

  ////一定周期で起動するスレッド/////
  thread1 = new Thread(new timerThread());
  thread1.start();
  ///////////////////////////////////
}

void draw()
{
  background(0);  
  writeSenValue();
  writeGraphYaw();
  writeGraphPitch();
  writeGraphRoll();  
  
  plotYaw();
  plotPitch();
  plotRoll();  

  draw3DGraphs(); 
}

void serialEvent(Serial p)
{
  if (port.available() > 22)
  {
    for (int i = 0; i < 23; i ++)
    {
      inByte[i] = port.read();
    }
    port.clear();

    for (int i = 3; i < 23; i = i + 2)
    {
      ad_new[i] = inByte[i] + (inByte[i+1]<<8);
      if (ad_new[i] >= 32767)
      {
        ad_new[i] = ad_new[i] - 65536;
      }
    }

    //センサ値を角加速度ベクトルに変換
    omega_vec[0] = radians(((float)(ad_new[11] - ad_base[11] ) )/16.4);
    omega_vec[1] = radians(((float)(ad_new[13] - ad_base[13] ) )/16.4);
    omega_vec[2] = radians(((float)(ad_new[15] - ad_base[15] ) )/16.4);
    
    //センサ値を加速度ベクトルに変換
    acc_vec[0]   = (ad_base[3] - ad_new[3])/2048.0; 
    acc_vec[1]   = (ad_base[5] - ad_new[5])/2048.0; 
    acc_vec[2]   = (ad_base[7] - ad_new[7])/2048.0;  
    
    //センサ値を地磁気ベクトルに変換
    mag_vec[0]   = ad_new[17] * 0.3;  
    mag_vec[1]   = ad_new[19] * 0.3; 
    mag_vec[2]   = ad_new[21] * 0.3;
    
    //加速度センサ値をLPFにかける
    acc_LPF_vec[0]     = gain_LPF*acc_LPF_vec[0]+(1.0-gain_LPF)*acc_vec[0];   
    acc_LPF_vec[1]     = gain_LPF*acc_LPF_vec[1]+(1.0-gain_LPF)*acc_vec[1];
    acc_LPF_vec[2]     = gain_LPF*acc_LPF_vec[2]+(1.0-gain_LPF)*acc_vec[2];
 
    //地磁気ベクトルを補正
    estimated_sphere_param = leastSquaresSpherePramUpdate(mag_vec[0], mag_vec[1], mag_vec[2]);
    mag_corrected_vec[0] = mag_vec[0] - estimated_sphere_param[1];
    mag_corrected_vec[1] = mag_vec[1] - estimated_sphere_param[2]; 
    mag_corrected_vec[2] = mag_vec[2] - estimated_sphere_param[3];      
    
    //地磁気センサ値をLPFにかける
    mag_cor_LPF_vec[0] = gain_LPF*mag_cor_LPF_vec[0] + (1.0-gain_LPF)*mag_corrected_vec[0];
    mag_cor_LPF_vec[1] = gain_LPF*mag_cor_LPF_vec[1] + (1.0-gain_LPF)*mag_corrected_vec[1];
    mag_cor_LPF_vec[2] = gain_LPF*mag_cor_LPF_vec[2] + (1.0-gain_LPF)*mag_corrected_vec[2];
       
    //クォータニオンの更新と姿勢行列の導出
    quat_3DOF = qAdd(quat_3DOF, qSMul(dt,diffQuat(quat_3DOF ,omega_vec)));
    quat_3DOF = qNormalize(quat_3DOF);
    rotMatrix_3DOF = quat2RotMatrix(quat_3DOF);
    
    //ジャイロの積分によるオイラー角の導出
    ZYX_euler_3DOF[0] = rotMatrix2ZYXYaw(rotMatrix_3DOF);
    ZYX_euler_3DOF[1] = rotMatrix2ZYXPitch(rotMatrix_3DOF);
    ZYX_euler_3DOF[2] = rotMatrix2ZYXRoll(rotMatrix_3DOF);  
    
    //ノルムの計算
    acc_norm = sqrt(acc_vec[0]*acc_vec[0]+acc_vec[1]*acc_vec[1]+acc_vec[2]*acc_vec[2]);
    mag_norm = sqrt(mag_vec[0]*mag_vec[0]+mag_vec[1]*mag_vec[1]+mag_vec[2]*mag_vec[2]);
    mag_cor_norm  = sqrt(mag_corrected_vec[0]*mag_corrected_vec[0]+
                         mag_corrected_vec[1]*mag_corrected_vec[1]+
                         mag_corrected_vec[2]*mag_corrected_vec[2]);
    
    //加速度ベクトルからroll角を導出    
    roll_accmag    = rangePi2mPi(atan2(acc_LPF_vec[1], acc_LPF_vec[2]) - PI);

    //加速度ベクトルからpitch角を導出
    pitch_accmag    =  rangePi2mPi(atan2(acc_LPF_vec[0], sqrt(acc_LPF_vec[2]*acc_LPF_vec[2]+acc_LPF_vec[1]*acc_LPF_vec[1] ) ));
  
    //6DOFの出力の計算
    float[] quat_yaw   = new float[4]; 
    float[] quat_pitch = new float[4];
    float[] quat_roll  = new float[4];
    quat_yaw[0] = cos(ZYX_euler_3DOF[0]/2.0); 
    quat_yaw[1] = 0.0;
    quat_yaw[2] = 0.0;
    quat_yaw[3] = sin(ZYX_euler_3DOF[0]/2.0); 
    
    quat_pitch[0] = cos(pitch_accmag/2.0);
    quat_pitch[1] = 0.0;
    quat_pitch[2] = sin(pitch_accmag/2.0);
    quat_pitch[3] = 0.0;
    
    quat_roll[0] = cos(roll_accmag/2.0);
    quat_roll[1] = sin(roll_accmag/2.0);
    quat_roll[2] = 0.0; 
    quat_roll[3] = 0.0;
    
    //相補フィルター(6DOF)
    quat_6DOF = qAdd( qSMul(gain_comp      , qAdd(quat_6DOF, qSMul(dt,diffQuat(quat_6DOF, omega_vec)))), 
                      qSMul((1.0-gain_comp), qMul(qMul(quat_roll,quat_pitch), quat_yaw))
                     );
    quat_6DOF = qNormalize(quat_6DOF); 
    rotMatrix_6DOF = quat2RotMatrix(quat_6DOF);
    
    ZYX_euler_6DOF[0] = rotMatrix2ZYXYaw(rotMatrix_6DOF);
    ZYX_euler_6DOF[1] = rotMatrix2ZYXPitch(rotMatrix_6DOF);
    ZYX_euler_6DOF[2] = rotMatrix2ZYXRoll(rotMatrix_6DOF);  
    
    //地磁気ベクトルからyaw角を算出
    quat_pitch[0] = cos(ZYX_euler_6DOF[1]/2.0);
    quat_pitch[1] = 0.0;
    quat_pitch[2] = sin(ZYX_euler_6DOF[1]/2.0);
    quat_pitch[3] = 0.0;
    
    quat_roll[0] = cos(ZYX_euler_6DOF[2]/2.0);
    quat_roll[1] = sin(ZYX_euler_6DOF[2]/2.0);
    quat_roll[2] = 0.0; 
    quat_roll[3] = 0.0;
   
   
    float[] quat_tmp = new float[4];
    quat_tmp = qMul(quat_roll,quat_pitch);
    float[] quat_mag = new float[4]; 
    quat_mag[0] = 0.0;
    quat_mag[1] = mag_cor_LPF_vec[0]; 
    quat_mag[2] = mag_cor_LPF_vec[1];
    quat_mag[3] = mag_cor_LPF_vec[2];
    quat_mag = qMul( qMul(quat_tmp,quat_mag),qConj(quat_tmp));
    yaw_accmag = rangePi2mPi(atan2(quat_mag[1],quat_mag[2]));
    
    //相補フィルター(9DOF)
    quat_yaw[0] = cos(yaw_accmag/2.0); 
    quat_yaw[1] = 0.0;
    quat_yaw[2] = 0.0;
    quat_yaw[3] = sin(yaw_accmag/2.0); 

    quat_9DOF = qAdd( qSMul(gain_comp      , qAdd(quat_9DOF, qSMul(dt,diffQuat(quat_9DOF, omega_vec)))), 
                      qSMul((1.0-gain_comp), qMul(qMul(quat_roll,quat_pitch), quat_yaw))
                     );
  quat_9DOF = qNormalize(quat_9DOF); 
    rotMatrix_9DOF = quat2RotMatrix(quat_9DOF);
    
    ZYX_euler_9DOF[0] = rotMatrix2ZYXYaw(rotMatrix_9DOF);
    ZYX_euler_9DOF[1] = rotMatrix2ZYXPitch(rotMatrix_9DOF);
    ZYX_euler_9DOF[2] = rotMatrix2ZYXRoll(rotMatrix_9DOF);  

    updateGraphValue();
  }
}

float rangePi2mPi(float ang_rad)
{
  float ang_out;
  ang_out = (ang_rad + 2* PI) % (2.0*PI);
  if (ang_out > PI) ang_out -= (2.0*PI);
  return ang_out;
}

void mouseClicked() {
  save("processing_birdsEye.jpg");
}

// thread for 1 sec. timer
class timerThread implements Runnable { 
  public synchronized void run() {
    while (true) {
      try {
        Thread.sleep(100);
      } 
      catch (InterruptedException e) {
      }

      //println("===============");
      for (int i=0; i<= 499; i++)
      {
        magdrawX[i]= (float)magdrawX[i+1];
        magdrawY[i]= (float)magdrawY[i+1];
        magdrawZ[i]= (float)magdrawZ[i+1];
      }
      magdrawX[500] = (float)ad_new[17];
      magdrawY[500] = (float)ad_new[19];
      magdrawZ[500] = (float)ad_new[21];
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////










